#lang plai

;; Gramática para el árbol de sintaxis abstraca.
;; Versión con azúcar sintáctica. Se incluyen:
;; · Identificadores
;; · Números
;; · Booleanos
;; · Listas
;; · Operaciones n-arias
;; · Condicionales: if y cond
;; · Asignaciones locales: with, with* y rec
;; · Funciones y su aplicación.
;; · Cajas
(define-type BRCFWBAEL
   [idS (i symbol?)]
   [numS (n number?)]
   [boolS (b boolean?)]
   [listS (l ListaS?)]
   [opS (f procedure?) (args (listof BRCFWBAEL?))]
   [ifS (expr BRCFWBAEL?) (then-expr BRCFWBAEL?) (else-expr BRCFWBAEL?)]
   [condS (cases (listof Condition?))]
   [withS (bindings (listof bindingS?)) (body BRCFWBAEL?)]
   [withS* (bindings (listof bindingS?)) (body BRCFWBAEL?)]
   [recS (bindings (listof bindingS?)) (body BRCFWBAEL?)]
   [funS (params (listof symbol?)) (body BRCFWBAEL?)]
   [appS (fun-expr BRCFWBAEL?) (args (listof BRCFWBAEL?))]
   [newboxS (expr BRCFWBAEL?)]
   [setboxS (expr BRCFWBAEL?) (value BRCFWBAEL?)]
   [openboxS (expr BRCFWBAEL?)]
   [seqnS (actions (listof BRCFWBAEL?))])

;; Versión desendulzada de la gramática RRCFWBAEL
(define-type BRCFBAEL
   [id (i symbol?)]
   [num (n number?)]
   [bool (b boolean?)]
   [lisT (l Lista?)]
   [op (f procedure?) (args (listof BRCFBAEL?))]
   [iF (expr BRCFBAEL?) (then-expr BRCFBAEL?) (else-expr BRCFBAEL?)]
   [fun (params (listof symbol?)) (body BRCFBAEL?)]
   [rec (bindings (listof binding?)) (body BRCFBAEL?)]
   [app (fun-expr BRCFBAEL?) (args (listof BRCFBAEL?))]
   [newbox (expr BRCFBAEL?)]
   [setbox (expr BRCFBAEL?) (value BRCFBAEL?)]
   [openbox (expr BRCFBAEL?)]
   [seqn (actions (listof BRCFBAEL?))])

;; Tipo ListaS que representa listas endulzadas.
(define-type ListaS
   [emptyLS]
   [consLS (expr BRCFWBAEL?) (xs BRCFWBAEL?)])

;; Tipo Lista que representa listas desendulzadas.
(define-type Lista
   [emptyL]
   [consL (expr BRCFBAEL?) (xs BRCFBAEL?)])

;; Tipo bindingS que representa bindings endulzados.
;; Se agrega esto, pues rec no está endulzado.
(define-type BindingS
   [bindingS (name symbol?) (value BRCFWBAEL?)])

;; Tipo bindings desendulzados.
(define-type Binding
   [binding (name symbol?) (value BRCFBAEL?)])

;; Para las condiciones de cond.
(define-type Condition
	[condition (expr BRCFWBAEL?) (then-expr BRCFWBAEL?)]
	[else-cond (else-expr BRCFWBAEL?)])

;; Ambientes recursivos.
(define-type Env
	[mtSub]
	[aSub (name symbol?) (location number?) (env Env?)])

;; Los resultados incluyen:
;; · Números
;; · Booleanos
;; · Funciones (Cerraduras)
;; · Listas
(define-type BRCFBAEL-Value
	[numV (n number?)]
	[boolV (b boolean?)]
	[closureV (params (listof symbol?)) (body BRCFBAEL?) (ds Env?)]
   [listV (l ListaVal?)]
   [boxV (location number?)])

;; Para representar listas como valor
(define-type ListaVal
   [emptyV]
   [consV (expr BRCFBAEL-Value?) (xs BRCFBAEL-Value?)])

;; Store
(define-type Store
   [mtSto]
   [aSto (location number?) (value BRCFBAEL-Value?) (store Store?)])

;; Valor con dirección del Store
(define-type Value*Store
   [v*s (value BRCFBAEL-Value?) (store Store?)])
